@@ -2,9 +2,9 @@ import os
 import random
 
 import imagesize
-import matplotlib.pyplot as plt
 import numpy as np
 import pandas as pd
+from PIL import Image
 from tiatoolbox.dataloader import wsireader
 
 from ..helper import helper
@@ -245,7 +245,8 @@ class ReadFromImageFile(Reader):
     def load_from_row(self, row, path_to_image):
         path = expression_to_string_value(self.expression, self.interpreter, row)
         path_to_file = os.path.join(path_to_image, path)
-        img = plt.imread(path_to_file)
+        # img = plt.imread(path_to_file)
+        img = np.asarray(Image.open(path_to_file))
         if len(img.shape) == 3 and img.shape[-1] == 4:
             img = img[..., :3]
         if self.return_image_size is not None:
