@@ -111,6 +111,8 @@ class HistoDataset(torch.utils.data.Dataset):
             else:
                 return self.feature_readers[feature_pos](self.df, self.path_to_dataset)
 
+    """
+
     def preload(self, batch_size=128):
         self.preloaded_data = {}
         self.preloaded_features = {}
@@ -141,6 +143,24 @@ class HistoDataset(torch.utils.data.Dataset):
                         )
                 else:
                     self.preloaded_features[k] = feat[k]
+    """
+
+    def preload(self, batch_size=128):
+        self.preloaded_data = {}
+        self.preloaded_features = {}
+        for i in range(0, len(self.df), batch_size):
+            data, features = self.__load__(self.df.iloc[i : i + batch_size].index)
+            for k in data:
+                if k in self.preloaded_data:
+                    self.preloaded_data[k].extend(data[k])
+                else:
+                    self.preloaded_data[k] = data[k]
+
+            for k in features:
+                if k in self.preloaded_data:
+                    self.preloaded_features[k].extend(features[k])
+                else:
+                    self.preloaded_features[k] = features[k]
 
     def __load__(self, idx):
         # get asked row
@@ -197,10 +217,16 @@ class HistoDataset(torch.utils.data.Dataset):
         else:
             data = {}
             for k in self.preloaded_data:
-                data[k] = self.preloaded_data[k][idx]
+                if isinstance(idx, list):
+                    data[k] = [self.preloaded_data[k][_idx] for _idx in idx]
+                else:
+                    data[k] = self.preloaded_data[k][idx]
             features = {}
             for k in self.preloaded_features:
-                features[k] = self.preloaded_features[k][idx]
+                if isinstance(idx, list):
+                    features[k] = [self.preloaded_features[k][_idx] for _idx in idx]
+                else:
+                    features[k] = self.preloaded_features[k][idx]
 
         # da transforms
         data = self.da_transfs(data, self.__get_hash__(idx))
@@ -212,30 +238,39 @@ class HistoDataset(torch.utils.data.Dataset):
         if self.return_pytorch_tensor:
             for key in data:
                 if isinstance(idx, list):
-                    is_numpy_array = isinstance(data[key][0], np.ndarray)
-                else:
-                    is_numpy_array = isinstance(data[key], np.ndarray)
-                data[key] = torch.as_tensor(data[key])
-
-                if is_numpy_array and (data[key].shape[-1] == 3 or data[key].shape[-1] == 1):
-                    if isinstance(idx, list) and len(data[key].shape) == 4:
-                        data[key] = data[key].permute(0, 3, 1, 2)
-                    elif not isinstance(idx, list) and len(data[key].shape) == 3:
-                        data[key] = data[key].permute(2, 0, 1)
+                    if isinstance(data[key], list):
+                        if not torch.is_tensor(data[key][0]):
+                            for i, d in enumerate(data[key]):
+                                if isinstance(d, np.ndarray):
+                                    data[key][i] = torch.as_tensor(d.copy())
+                                else:
+                                    data[key][i] = torch.as_tensor(d)
+                        data[key] = torch.stack(data[key])
+                    elif not torch.is_tensor(data[key]):
+                        data[key] = torch.as_tensor(data[key])
+                elif not torch.is_tensor(data[key]):
+                    if isinstance(data[key], np.ndarray):
+                        data[key] = torch.as_tensor(data[key].copy())
+                    else:
+                        data[key] = torch.as_tensor(data[key])
 
             for key in features:
                 if isinstance(idx, list):
-                    is_numpy_array = isinstance(features[key][0], np.ndarray)
-                else:
-                    is_numpy_array = isinstance(features[key], np.ndarray)
-                features[key] = torch.as_tensor(features[key])
-                if is_numpy_array and (
-                    features[key].shape[-1] == 3 or features[key].shape[-1] == 1
-                ):
-                    if isinstance(idx, list) and len(features[key].shape) == 4:
-                        features[key] = features[key].permute(0, 3, 1, 2)
-                    elif not isinstance(idx, list) and len(features[key].shape) == 3:
-                        features[key] = features[key].permute(2, 0, 1)
+                    if isinstance(features[key], list):
+                        if not torch.is_tensor(features[key][0]):
+                            for i, d in enumerate(features[key]):
+                                if isinstance(d, np.ndarray):
+                                    features[key][i] = torch.as_tensor(d.copy())
+                                else:
+                                    features[key][i] = torch.as_tensor(d)
+                        features[key] = torch.stack(features[key])
+                    elif not torch.is_tensor(features[key]):
+                        features[key] = torch.as_tensor(features[key])
+                elif not torch.is_tensor(features[key]):
+                    if isinstance(features[key], np.ndarray):
+                        features[key] = torch.as_tensor(features[key].copy())
+                    else:
+                        features[key] = torch.as_tensor(features[key])
 
         dic = {}
         dic.update(data)
